@@ -28,7 +28,7 @@
  * SUCH DAMAGE.
  *
  *
- *	$Id: im_m17nlib.c,v 1.6 2004/10/21 21:02:34 seiichi Exp $
+ *	$Id: im_m17nlib.c,v 1.18 2005/03/03 07:24:35 seiichi Exp $
  */
 
 #include  <X11/keysym.h>	/* XK_xxx */
@@ -47,7 +47,10 @@
 #define  IM_M17NLIB_DEBUG 1
 #endif
 
-#define  MAX_BYTES_PER_CHAR  4	/* FIXME */
+#define  MAX_BYTES_PER_CHAR	4	/* FIXME */
+#define  MAX_BYTES_ESC_SEQUEACE	5	/* FIXME */
+
+#define  MAX_BYTES(n) (((n) * MAX_BYTES_PER_CHAR) + MAX_BYTES_ESC_SEQUEACE + 1)
 
 typedef struct im_m17nlib
 {
@@ -70,7 +73,7 @@ static int  ref_count = 0 ;
 static int  initialized = 0 ;
 static mkf_parser_t *  parser_ascii = NULL ;
 /* mlterm internal symbols */
-static x_im_export_syms_t *  mlterm_syms = NULL ;
+static x_im_export_syms_t *  syms = NULL ;
 
 
 /* --- static functions --- */
@@ -148,6 +151,7 @@ xksym_to_msymbol(
 		if( is_shift && ( 'a' <= buf[0] && buf[0] <= 'z'))
 		{
 			buf[0] += ( 'A' - 'a') ;
+			is_shift = 0 ;
 		}
 
 		return  msymbol( buf) ;
@@ -240,7 +244,7 @@ find_input_method(
 	if( ! ( im_list = mdatabase_list( msymbol( "input-method") ,
 					  Mnil , Mnil , Mnil)))
 	{
-		kik_error_printf( "Cound not get available input method from m17n database.\n") ;
+		kik_error_printf( "There are no available input methods.\n") ;
 		return  0 ;
 	}
 
@@ -297,6 +301,47 @@ find_input_method(
 	return  result ;
 }
 
+static void
+commit(
+	im_m17nlib_t *  m17nlib ,
+	MText *  text
+	)
+{
+	u_char *  buf = NULL ;
+	u_int  num_of_chars ;
+	int  filled_len ;
+
+	if( ( num_of_chars = mtext_len( text)))
+	{
+		if( ! ( buf = alloca( MAX_BYTES(num_of_chars))))
+		{
+		#ifdef  DEBUG
+			kik_warn_printf( KIK_DEBUG_TAG , " alloca failed\n") ;
+		#endif
+		}
+	}
+
+	if( buf)
+	{
+		mconv_reset_converter( m17nlib->mconverter) ;
+		mconv_rebind_buffer(
+				m17nlib->mconverter , buf ,
+				MAX_BYTES(num_of_chars)) ;
+		filled_len = mconv_encode( m17nlib->mconverter , text) ;
+
+		if( filled_len == -1)
+		{
+			kik_error_printf( "Could not convert the encoding of committed characters. [error code: %d]\n" , merror_code) ;
+		}
+		else
+		{
+			(*m17nlib->im.listener->write_to_term)(
+					m17nlib->im.listener->self ,
+					buf , filled_len) ;
+		}
+	}
+}
+
 static void
 set_candidate(
 	im_m17nlib_t *  m17nlib ,
@@ -313,7 +358,7 @@ set_candidate(
 		return ;
 	}
 
-	if( ! ( buf = alloca( num_of_chars * MAX_BYTES_PER_CHAR + 1)))
+	if( ! ( buf = alloca( MAX_BYTES(num_of_chars))))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG , " alloca failed\n") ;
@@ -321,17 +366,21 @@ set_candidate(
 		return ;
 	}
 
+	mconv_reset_converter( m17nlib->mconverter) ;
 	mconv_rebind_buffer( m17nlib->mconverter , buf ,
-			     num_of_chars * MAX_BYTES_PER_CHAR + 1) ;
+			     MAX_BYTES(num_of_chars)) ;
 	filled_len = mconv_encode( m17nlib->mconverter , candidate) ;
 
-	if( filled_len > 0)
+	if( filled_len == -1)
 	{
-		buf[filled_len] = '\0' ;
-		(*m17nlib->im.cand_screen->set)( m17nlib->im.cand_screen ,
-						 m17nlib->parser_term ,
-						 buf , idx) ;
+		kik_error_printf( "Could not convert the encoding of characters in candidates. [error code: %d]\n" , merror_code) ;
+		return ;
 	}
+
+	buf[filled_len] = '\0' ;
+	(*m17nlib->im.cand_screen->set)( m17nlib->im.cand_screen ,
+					 m17nlib->parser_term ,
+					 buf , idx) ;
 }
 
 static void
@@ -356,8 +405,7 @@ preedit_changed(
 
 	if( m17nlib->im.preedit.chars)
 	{
-		(*mlterm_syms->ml_str_delete)(
-					m17nlib->im.preedit.chars ,
+		(*syms->ml_str_delete)( m17nlib->im.preedit.chars ,
 					m17nlib->im.preedit.num_of_chars) ;
 		m17nlib->im.preedit.chars = NULL ;
 	}
@@ -379,7 +427,7 @@ preedit_changed(
 		goto  draw ;
 	}
 
-	if( ! ( buf = alloca( num_of_chars  * MAX_BYTES_PER_CHAR)))
+	if( ! ( buf = alloca( MAX_BYTES(num_of_chars))))
 	{
 	#ifdef  DEBUG
 		kik_warn_printf( KIK_DEBUG_TAG , " alloca failed\n") ;
@@ -387,13 +435,15 @@ preedit_changed(
 		return ;
 	}
 
-	mconv_rebind_buffer( m17nlib->mconverter ,
-			     buf , num_of_chars * MAX_BYTES_PER_CHAR) ;
+	mconv_reset_converter( m17nlib->mconverter) ;
+	mconv_rebind_buffer( m17nlib->mconverter , buf ,
+			     MAX_BYTES(num_of_chars)) ;
 	filled_len = mconv_encode( m17nlib->mconverter ,
 				   m17nlib->input_context->preedit) ;
 
 	if( filled_len == -1)
 	{
+		kik_error_printf( "Could not convert the preedit string to terminal encoding. [%d]\n" , merror_code) ;
 		return ;
 	}
 
@@ -402,10 +452,10 @@ preedit_changed(
 	 * allocate im.preedit.chars
 	 */
 
-	if( ! ( m17nlib->im.preedit.chars = malloc( sizeof(ml_char_t) * num_of_chars)))
+	if( ! ( m17nlib->im.preedit.chars = calloc( num_of_chars , sizeof(ml_char_t))))
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG , " malloc failed\n") ;
+		kik_warn_printf( KIK_DEBUG_TAG , " calloc failed\n") ;
 	#endif
 		return ;
 	}
@@ -417,17 +467,15 @@ preedit_changed(
 	 */
 
 	p = m17nlib->im.preedit.chars ;
-	(*mlterm_syms->ml_str_init)( p , m17nlib->im.preedit.num_of_chars);
+	(*syms->ml_str_init)( p , m17nlib->im.preedit.num_of_chars);
 
 	(*m17nlib->parser_term->init)( m17nlib->parser_term) ;
 	(*m17nlib->parser_term->set_str)( m17nlib->parser_term ,
 					  (u_char*) buf ,
 					  filled_len) ;
 
-	m17nlib->im.preedit.segment_offset =
-					m17nlib->input_context->candidate_from ;
-	m17nlib->im.preedit.cursor_offset =
-					m17nlib->input_context->cursor_pos ;
+	m17nlib->im.preedit.segment_offset = m17nlib->input_context->candidate_from ;
+	m17nlib->im.preedit.cursor_offset = m17nlib->input_context->cursor_pos ;
 
 	while( (*m17nlib->parser_term->next_char)( m17nlib->parser_term , &ch))
 	{
@@ -450,7 +498,8 @@ preedit_changed(
 			}
 		}
 
-		if( m17nlib->input_context->candidate_from <= pos &&
+		if( m17nlib->input_context->candidate_list &&
+		    m17nlib->input_context->candidate_from <= pos &&
 		    m17nlib->input_context->candidate_to > pos)
 		{
 			fg_color = ML_BG_COLOR ;
@@ -465,13 +514,13 @@ preedit_changed(
 
 		if( is_comb)
 		{
-			if( (*mlterm_syms->ml_char_combine)( p - 1 , ch.ch ,
-							     ch.size , ch.cs ,
-							     is_biwidth ,
-							     is_comb ,
-							     fg_color ,
-							     bg_color ,
-							     0 , is_underline))
+			if( (*syms->ml_char_combine)( p - 1 , ch.ch ,
+						      ch.size , ch.cs ,
+						      is_biwidth ,
+						      is_comb ,
+						      fg_color ,
+						      bg_color ,
+						      0 , is_underline))
 			{
 				pos++ ;
 				continue;
@@ -482,15 +531,15 @@ preedit_changed(
 			 */
 		}
 
-		if( (*mlterm_syms->ml_is_msb_set)( ch.cs))
+		if( (*syms->ml_is_msb_set)( ch.cs))
 		{
 			SET_MSB( ch.ch[0]) ;
 		}
 
-		(*mlterm_syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
-					     is_biwidth , is_comb ,
-					     fg_color , bg_color ,
-					     0 , is_underline) ;
+		(*syms->ml_char_set)( p , ch.ch , ch.size , ch.cs ,
+				      is_biwidth , is_comb ,
+				      fg_color , bg_color ,
+				      0 , is_underline) ;
 
 		pos++ ;
 		p++ ;
@@ -499,13 +548,10 @@ preedit_changed(
 
 draw:
 
-	(*m17nlib->im.listener->draw_preedit_str)(
-					m17nlib->im.listener->self ,
-					m17nlib->im.preedit.chars ,
-					m17nlib->im.preedit.filled_len ,
-					m17nlib->im.preedit.cursor_offset) ;
-
-	m17nlib->input_context->candidate_to = 0 ; /* XXX: bug of m17n-lib? */
+	(*m17nlib->im.listener->draw_preedit_str)( m17nlib->im.listener->self ,
+						   m17nlib->im.preedit.chars ,
+						   m17nlib->im.preedit.filled_len ,
+						   m17nlib->im.preedit.cursor_offset) ;
 }
 
 static void
@@ -529,15 +575,13 @@ candidates_changed(
 	{
 		if( m17nlib->im.cand_screen)
 		{
-			(*m17nlib->im.cand_screen->delete)(
-						m17nlib->im.cand_screen) ;
+			(*m17nlib->im.cand_screen->delete)( m17nlib->im.cand_screen) ;
 			m17nlib->im.cand_screen = NULL ;
 		}
 
 		if( m17nlib->im.stat_screen)
 		{
-			(*m17nlib->im.stat_screen->show)(
-						m17nlib->im.stat_screen) ;
+			(*m17nlib->im.stat_screen->show)( m17nlib->im.stat_screen) ;
 		}
 
 		return ;
@@ -561,22 +605,26 @@ candidates_changed(
 	kik_debug_printf( KIK_DEBUG_TAG " number of candidates: %d\n", num_of_candidates) ;
 #endif
 
-	(*m17nlib->im.listener->get_spot)(
-				m17nlib->im.listener->self ,
-				m17nlib->im.preedit.chars ,
-				m17nlib->im.preedit.segment_offset ,
-				&x , &y) ;
+	(*m17nlib->im.listener->get_spot)( m17nlib->im.listener->self ,
+					   m17nlib->im.preedit.chars ,
+					   m17nlib->im.preedit.segment_offset ,
+					   &x , &y) ;
 
 	if( m17nlib->im.cand_screen == NULL)
 	{
-		int  is_vertical ;
-		u_int  line_height ;
+		int  is_vertical_direction = 0 ;
 
-		if( ! ( m17nlib->im.cand_screen = (*mlterm_syms->x_im_candidate_screen_new)(
+		if( strcmp( msymbol_name( m17nlib->input_method->name) , "anthy") == 0)
+		{
+			is_vertical_direction = 1 ;
+		}
+
+		if( ! ( m17nlib->im.cand_screen = (*syms->x_im_candidate_screen_new)(
 				(*m17nlib->im.listener->get_win_man)(m17nlib->im.listener->self) ,
 				(*m17nlib->im.listener->get_font_man)(m17nlib->im.listener->self) ,
 				(*m17nlib->im.listener->get_color_man)(m17nlib->im.listener->self) ,
 				(*m17nlib->im.listener->is_vertical)(m17nlib->im.listener->self) ,
+				is_vertical_direction ,
 				(*m17nlib->im.listener->get_line_height)(m17nlib->im.listener->self) ,
 				x , y)))
 		{
@@ -600,8 +648,6 @@ candidates_changed(
 		return ;
 	}
 
-	(*m17nlib->im.cand_screen->set_spot)( m17nlib->im.cand_screen , x , y) ;
-
 	group = m17nlib->input_context->candidate_list ;
 
 	if( mplist_key( group) == Mtext)
@@ -615,8 +661,7 @@ candidates_changed(
 			{
 				MText *  text ;
 				text = mtext() ;
-				mtext_cat_char( text ,
-						mtext_ref_char( mplist_value( group) , i)) ;
+				mtext_cat_char( text , mtext_ref_char( mplist_value( group) , i)) ;
 				set_candidate( m17nlib , text , idx) ;
 				m17n_object_unref( text) ;
 				idx++ ;
@@ -625,9 +670,7 @@ candidates_changed(
 	}
 	else
 	{
-		for( idx = 0 ;
-		     mplist_key( group) != Mnil ;
-		     group = mplist_next( group))
+		for( idx = 0 ; mplist_key( group) != Mnil ; group = mplist_next( group))
 		{
 			for( candidate = mplist_value( group) ;
 			     mplist_key( candidate) != Mnil ;
@@ -640,14 +683,14 @@ candidates_changed(
 		}
 	}
 
-	(*m17nlib->im.cand_screen->select)(
-				m17nlib->im.cand_screen ,
-				m17nlib->input_context->candidate_index) ;
+	(*m17nlib->im.cand_screen->select)( m17nlib->im.cand_screen ,
+					    m17nlib->input_context->candidate_index) ;
+
+	(*m17nlib->im.cand_screen->set_spot)( m17nlib->im.cand_screen , x , y) ;
 
 	if( m17nlib->im.stat_screen)
 	{
-		(*m17nlib->im.stat_screen->hide)(
-					m17nlib->im.stat_screen) ;
+		(*m17nlib->im.stat_screen->hide)( m17nlib->im.stat_screen) ;
 	}
 }
 
@@ -733,7 +776,10 @@ key_event(
 		return  1 ;
 	}
 
-	mkey = xksym_to_msymbol( m17nlib , ksym , event->state) ;
+	if( ( mkey = xksym_to_msymbol( m17nlib , ksym , event->state)) == Mnil)
+	{
+		return  1 ;
+	}
 
 	if( minput_filter( m17nlib->input_context , mkey , Mnil))
 	{
@@ -752,39 +798,13 @@ key_event(
 
 	text = mtext() ;
 
-	if( minput_lookup( m17nlib->input_context ,
-			   Mnil , NULL , text) == 0)
+	if( minput_lookup( m17nlib->input_context , Mnil , NULL , text) == 0)
 	{
-		u_char *  buf = NULL ;
-		u_int  num_of_chars ;
-		int  filled_len ;
-
-		if( ( num_of_chars = mtext_len( text)))
+		if( mtext_len( text))
 		{
-			if( ! ( buf = alloca( num_of_chars * MAX_BYTES_PER_CHAR + 1)))
-			{
-			#ifdef  DEBUG
-				kik_warn_printf( KIK_DEBUG_TAG , " alloca failed\n") ;
-			#endif
-			}
+			commit( m17nlib , text) ;
+			ret = 0 ;
 		}
-
-		if( buf)
-		{
-			mconv_rebind_buffer(
-					m17nlib->mconverter , buf ,
-					num_of_chars * MAX_BYTES_PER_CHAR + 1) ;
-			filled_len = mconv_encode( m17nlib->mconverter , text) ;
-
-			if( filled_len > 0)
-			{
-				(*m17nlib->im.listener->write_to_term)(
-						m17nlib->im.listener->self ,
-						buf , filled_len) ;
-			}
-		}
-
-		ret = 0 ;
 	}
 
 	m17n_object_unref( text) ;
@@ -803,20 +823,19 @@ switch_mode(
 
 	m17nlib = (im_m17nlib_t*) im ;
 
-	minput_toggle( m17nlib->input_context) ;
-
 	(*m17nlib->im.listener->get_spot)( m17nlib->im.listener->self ,
 					   m17nlib->im.preedit.chars ,
 					   m17nlib->im.preedit.segment_offset ,
 					   &x , &y) ;
 
-	if( m17nlib->input_context->active)
+	if( ! m17nlib->input_context->active)
 	{
 		u_char  buf[50] ;
+		u_int  filled_len ;
 
 		if( m17nlib->im.stat_screen == NULL)
 		{
-			if( ! ( m17nlib->im.stat_screen = (*mlterm_syms->x_im_status_screen_new)(
+			if( ! ( m17nlib->im.stat_screen = (*syms->x_im_status_screen_new)(
 					(*m17nlib->im.listener->get_win_man)(m17nlib->im.listener->self) ,
 					(*m17nlib->im.listener->get_font_man)(m17nlib->im.listener->self) ,
 					(*m17nlib->im.listener->get_color_man)(m17nlib->im.listener->self) ,
@@ -831,22 +850,72 @@ switch_mode(
 			}
 		}
 
-		kik_snprintf( buf , sizeof( buf) , "m17nlib:%s:%s" ,
-			      msymbol_name(m17nlib->input_method->language) ,
-			      msymbol_name(m17nlib->input_method->name)) ;
+		mconv_reset_converter( m17nlib->mconverter) ;
+		mconv_rebind_buffer( m17nlib->mconverter , buf , sizeof( buf)) ;
+		filled_len = mconv_encode( m17nlib->mconverter ,
+					   m17nlib->input_context->status) ;
 
-		(*m17nlib->im.stat_screen->set)( m17nlib->im.stat_screen ,
-						 parser_ascii , buf) ;
+		if( filled_len == -1)
+		{
+			kik_error_printf( "Could not convert the encoding of characters for status. [%d]\n" , merror_code) ;
+		}
+		else
+		{
+			buf[filled_len] = 0 ;
+			(*m17nlib->im.stat_screen->set)( m17nlib->im.stat_screen ,
+							 m17nlib->parser_term , buf) ;
+		}
 	}
 	else
 	{
+		/*
+		 * commit the last preedit before deactivating the input
+		 * method.
+		 */
+		if( mtext_len( m17nlib->input_context->preedit))
+		{
+			commit( m17nlib , m17nlib->input_context->preedit) ;
+		}
+
+		/*
+		 * initialize the state of MinputContext.
+		 * <http://sf.net/mailarchive/message.php?msg_id=9958816>
+		 */
+		minput_filter( m17nlib->input_context , Mnil , Mnil);
+
+
+		/*
+		 * clear saved preedit
+		 */
+
+		if( m17nlib->im.preedit.chars)
+		{
+			(*syms->ml_str_delete)( m17nlib->im.preedit.chars ,
+						m17nlib->im.preedit.num_of_chars) ;
+			m17nlib->im.preedit.chars = NULL ;
+		}
+
+		m17nlib->im.preedit.num_of_chars = 0 ;
+		m17nlib->im.preedit.filled_len = 0 ;
+		m17nlib->im.preedit.segment_offset = 0 ;
+		m17nlib->im.preedit.cursor_offset = X_IM_PREEDIT_NOCURSOR ;
+
+
 		if( m17nlib->im.stat_screen)
 		{
 			(*m17nlib->im.stat_screen->delete)( m17nlib->im.stat_screen) ;
 			m17nlib->im.stat_screen = NULL ;
 		}
+
+		if( m17nlib->im.cand_screen)
+		{
+			(*m17nlib->im.cand_screen->delete)( m17nlib->im.cand_screen) ;
+			m17nlib->im.cand_screen = NULL ;
+		}
 	}
 
+	minput_toggle( m17nlib->input_context) ;
+
 	return  1 ;
 }
 
@@ -857,7 +926,7 @@ focused(
 {
 	im_m17nlib_t *  m17nlib ;
 
-	m17nlib =  (im_m17nlib_t*) im ;
+	m17nlib = (im_m17nlib_t*) im ;
 
 	if( m17nlib->im.cand_screen)
 	{
@@ -893,15 +962,15 @@ unfocused(
 /* --- global functions --- */
 
 x_im_t *
-im_new(
+im_m17nlib_new(
 	u_int64_t  magic ,
 	ml_char_encoding_t  term_encoding ,
 	x_im_export_syms_t *  export_syms ,
-	char *  param	/* <language>:<input method> */
+	char *  param ,	/* <language>:<input method> */
+	u_int  mod_ignore_mask
 	)
 {
 	im_m17nlib_t *  m17nlib ;
-	MInputMethod *  input_method ;
 	char *  encoding_name ;
 	MSymbol  encoding_sym ;
 
@@ -948,11 +1017,11 @@ im_new(
 			goto  error ;
 		}
 
-		mlterm_syms = export_syms ;
+		syms = export_syms ;
 
 		initialized = 1 ;
 
-		if( ! ( parser_ascii = (*mlterm_syms->ml_parser_new)( ML_ISO8859_1)))
+		if( ! ( parser_ascii = (*syms->ml_parser_new)( ML_ISO8859_1)))
 		{
 			goto  error ;
 		}
@@ -978,43 +1047,48 @@ im_new(
 
 	if( ! ( m17nlib->input_method = find_input_method( param)))
 	{
-		kik_error_printf( "Cound not find %s\n" , param) ;
+		kik_error_printf( "Could not find %s\n" , param) ;
 		goto  error ;
 	}
 
-	if( ! ( m17nlib->input_context = minput_create_ic(
-							m17nlib->input_method ,
-							NULL)))
+	if( ! ( m17nlib->input_context = minput_create_ic( m17nlib->input_method , NULL)))
 	{
-		kik_error_printf( "Cound not crate context for %s\n", param) ;
+		kik_error_printf( "Could not crate context for %s\n", param) ;
 		goto  error ;
 	}
 
-	encoding_name = (*mlterm_syms->ml_get_char_encoding_name)( term_encoding) ;
+	if( term_encoding == ML_EUCJISX0213)
+	{
+		encoding_name = (*syms->ml_get_char_encoding_name)( ML_EUCJP) ;
+	}
+	else
+	{
+		encoding_name = (*syms->ml_get_char_encoding_name)( term_encoding) ;
+	}
+
 
 	if( ( encoding_sym = mconv_resolve_coding( msymbol( encoding_name))) == Mnil)
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " Cound not resolve encoding name [%s]\n" , encoding_name) ;
+		kik_warn_printf( KIK_DEBUG_TAG " Could not resolve encoding name [%s]\n" , encoding_name) ;
 	#endif
 		goto  error ;
 	}
 
-	if( ! ( m17nlib->mconverter = mconv_buffer_converter( encoding_sym ,
-							      NULL , 0)))
+	if( ! ( m17nlib->mconverter = mconv_buffer_converter( encoding_sym , NULL , 0)))
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " Cound not create MConverter\n") ;
+		kik_warn_printf( KIK_DEBUG_TAG " Could not create MConverter\n") ;
 	#endif
 		goto  error ;
 	}
 
-	if( ! ( m17nlib->conv = (*mlterm_syms->ml_conv_new)( term_encoding)))
+	if( ! ( m17nlib->conv = (*syms->ml_conv_new)( term_encoding)))
 	{
 		goto  error ;
 	}
 
-	if( ! ( m17nlib->parser_term = (*mlterm_syms->ml_parser_new)( term_encoding)))
+	if( ! ( m17nlib->parser_term = (*syms->ml_parser_new)( term_encoding)))
 	{
 		goto  error ;
 	}
@@ -1089,7 +1163,7 @@ error:
 /* --- API for external tools --- */
 
 im_info_t *
-im_get_info(
+im_m17nlib_get_info(
 	char *  locale ,
 	char *  encoding
 	)
@@ -1120,12 +1194,12 @@ im_get_info(
 	result->id = strdup( "m17nlib") ;
 	result->name = strdup( "m17n library") ;
 	result->num_of_args = num_of_ims + 1;
-	if( ! ( result->args = malloc( sizeof(char*) * result->num_of_args)))
+	if( ! ( result->args = calloc( result->num_of_args , sizeof(char*))))
 	{
 		goto  error ;
 	}
 
-	if( ! ( result->readable_args = malloc( sizeof(char*) * result->num_of_args)))
+	if( ! ( result->readable_args = calloc( result->num_of_args , sizeof(char*))))
 	{
 		free( result->args) ;
 		goto  error ;
