@@ -19,7 +19,7 @@
  * Boston, MA  02111-1307  USA
  *
  *
- * $Id: im_scim_mod_if.c,v 1.8 2005/03/03 15:06:06 seiichi Exp $
+ * $Id: im_scim_mod_if.c,v 1.9 2005/11/09 16:01:49 seiichi Exp $
  *
  */
 
@@ -475,7 +475,8 @@ candidate_update(
 	}
 
 	if( ! (*scim->im.cand_screen->init)( scim->im.cand_screen ,
-					     num_of_candiate , 10)) /* 16? */
+					     num_of_candiate ,
+					     num_of_candiate))
 	{
 		(*scim->im.cand_screen->delete)( scim->im.cand_screen) ;
 		scim->im.cand_screen = NULL ;
