@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_iscii.h,v 1.2 2003/02/02 04:43:04 arakiken Exp $
+ *	$Id: ml_iscii.h,v 1.3 2004/12/13 09:31:54 h_minami Exp $
  */
 
 #ifndef  __ML_ISCII_H__
@@ -29,7 +29,7 @@ typedef enum  ml_iscii_lang_type
 	ISCIILANG_TAMIL ,
 	ISCIILANG_TELUGU ,
 
-	MAX_ISCIILANG ,
+	MAX_ISCIILANG
 
 } ml_iscii_lang_type_t ;
 
