@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_line.c,v 1.30 2003/09/14 16:44:28 arakiken Exp $
+ *	$Id: ml_line.c,v 1.34 2004/03/03 12:45:06 arakiken Exp $
  */
 
 #include  "ml_line.h"
@@ -1630,7 +1630,7 @@ end:
 }
 
 /*
- * this should be called after ml_line_iscii_visaul()
+ * This should be called after ml_line_iscii_visaul()
  */
 int
 ml_iscii_convert_logical_char_index_to_visual(
@@ -1676,15 +1676,17 @@ ml_iscii_convert_logical_char_index_to_visual(
 		visual_char_index ++ ;
 	}
 
-	while( ml_char_is_null( line->chars + count))
+	do
 	{
-		if( ++ count >= line->num_of_filled_chars)
+		if( count >= line->num_of_filled_chars || ! ml_char_is_null( line->chars + count))
 		{
 			break ;
 		}
-		
+
+		count ++ ;
 		visual_char_index ++ ;
 	}
+	while( 1) ;
 
 end:
 	if( visual_char_index >= line->num_of_filled_chars)
