@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_term_manager.c,v 1.18 2003/09/15 06:14:18 arakiken Exp $
+ *	$Id: ml_term_manager.c,v 1.20 2004/11/21 12:56:50 seiichi Exp $
  */
 
 #include  "ml_term_manager.h"
@@ -33,6 +33,16 @@ sig_child(
 {
 	int  count ;
 
+	if( pid <= 0)
+	{
+		/*
+		 * Note:
+		 * If term->pty is NULL, ml_term_get_child_pid() returns -1.
+		 */
+
+		return ;
+	}
+
 	for( count = 0 ; count < num_of_terms ; count ++)
 	{
 		if( pid == ml_term_get_child_pid( terms[count]))
