@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_vt100_parser.c,v 1.45 2005/02/23 07:47:01 h_minami Exp $
+ *	$Id: ml_vt100_parser.c,v 1.51 2006/03/21 09:54:13 seiichi Exp $
  */
 
 #include  "ml_vt100_parser.h"
@@ -29,8 +29,8 @@
 #define  CTLKEY_LF	0x0a
 #define  CTLKEY_VT	0x0b
 #define  CTLKEY_CR	0x0d
-#define  CTLKEY_SO      0x0e
-#define  CTLKEY_SI      0x0f
+#define  CTLKEY_SO	0x0e
+#define  CTLKEY_SI	0x0f
 #define  CTLKEY_ESC	0x1b
 
 #define  CURRENT_STR_P(vt100_parser)  (&vt100_parser->seq[(vt100_parser)->len - (vt100_parser)->left])
@@ -173,7 +173,7 @@ receive_bytes(
 			}
 
 			if( ( vt100_parser->log_file =
-				open( path , O_CREAT | O_TRUNC | O_WRONLY , 0600)) == -1)
+				open( path , O_CREAT | O_APPEND | O_WRONLY , 0600)) == -1)
 			{
 				free( path) ;
 
@@ -187,7 +187,12 @@ receive_bytes(
 	#if  1
 		fsync( vt100_parser->log_file) ;
 	#endif
-	}
+	} else {
+		if ( vt100_parser->log_file != -1) {
+			close( vt100_parser->log_file) ;
+			vt100_parser->log_file = -1;
+		}
+        }
 
 end:
 	vt100_parser->len = ( vt100_parser->left += ret) ;
@@ -467,6 +472,7 @@ save_cursor(
 	vt100_parser->saved_is_bold = vt100_parser->is_bold ;
 	vt100_parser->saved_is_underlined = vt100_parser->is_underlined ;
 	vt100_parser->saved_is_reversed = vt100_parser->is_reversed ;
+	vt100_parser->saved_cs = vt100_parser->cs ;
 
 	ml_screen_save_cursor( vt100_parser->screen) ;
 }
@@ -476,12 +482,42 @@ restore_cursor(
 	ml_vt100_parser_t *  vt100_parser
 	)
 {
-	if( vt100_parser->is_saved){
+	if( vt100_parser->is_saved)
+	{
 		vt100_parser->fg_color = vt100_parser->saved_fg_color ;
 		vt100_parser->bg_color = vt100_parser->saved_bg_color ;
 		vt100_parser->is_bold = vt100_parser->saved_is_bold ;
 		vt100_parser->is_underlined = vt100_parser->saved_is_underlined ;
 		vt100_parser->is_reversed = vt100_parser->saved_is_reversed ;
+		if( IS_ENCODING_BASED_ON_ISO2022(vt100_parser->encoding))
+		{
+			if( vt100_parser->saved_cs != vt100_parser->cs)
+			{
+				if( vt100_parser->saved_cs == DEC_SPECIAL)
+				{
+					/* force grapchics mode by sending \E(0 to current parser*/
+					u_char  DEC_SEQ[] = { CTLKEY_ESC, '(', '0'} ;
+					mkf_char_t  ch ;
+					mkf_parser_t *  parser;
+					
+					ml_init_encoding_parser( vt100_parser) ;
+					parser = vt100_parser->cc_parser;
+					(*parser->set_str)( parser, DEC_SEQ, sizeof(DEC_SEQ)) ;
+					(*parser->next_char)( parser, &ch) ;
+				}
+			}
+		}
+		else
+		{
+			/* XXX: what to do for g0/g1? */
+			if( vt100_parser->saved_cs == DEC_SPECIAL){
+				vt100_parser->is_dec_special_in_gl = 1;
+			}
+			else
+			{
+				vt100_parser->is_dec_special_in_gl = 0;
+			}
+		}
 	}
 	ml_screen_restore_cursor( vt100_parser->screen) ;
 }
@@ -1114,7 +1150,7 @@ parse_vt100_escape_sequence(
 					}
 					if( '0' <= *str_p && *str_p <= '9')
 					{
-						u_char  digit[DIGIT_STR_LEN(int)] ;
+						char  digit[DIGIT_STR_LEN(int)] ;
 						int  count ;
 
 						digit[0] = *str_p ;
@@ -1328,26 +1364,26 @@ parse_vt100_escape_sequence(
 					#endif
 						else if( ps[0] == 1047)
 						{
-							// if( !titeInhibit)
+							/* if( !titeInhibit)*/
 							save_cursor( vt100_parser) ;
 							ml_screen_use_alternative_edit(
 								vt100_parser->screen) ;
-							//
+							/* */
 						}
 						else if( ps[0] == 1048)
 						{
-							// if( !titeInhibit)
+							/* if( !titeInhibit)*/
 							save_cursor( vt100_parser) ;
-							//
+							/* */
 						}
 						else if( ps[0] == 1049)
 						{
-							// if( !titeInhibit)
+							/* if( !titeInhibit)*/
 							save_cursor( vt100_parser) ;
 							ml_screen_use_alternative_edit(
 								vt100_parser->screen) ;
 							clear_display_all( vt100_parser) ;
-							//
+							/* */
 						}
 						else
 						{
@@ -1496,26 +1532,26 @@ parse_vt100_escape_sequence(
 					#endif
 						else if( ps[0] == 1047)
 						{
-							// if( !titeInhibit)
+							/* if( !titeInhibit)*/
 							clear_display_all( vt100_parser) ;
 							ml_screen_use_normal_edit(
 								vt100_parser->screen) ;
 							restore_cursor( vt100_parser) ;
-							//
+							/* */
 						}
 						else if( ps[0] == 1048)
 						{
-							// if( !titeInhibit)
+							/* if( !titeInhibit)*/
 							restore_cursor( vt100_parser) ;
-							//
+							/* */
 						}
 						else if( ps[0] == 1049)
 						{
-							// if( !titeInhibit)
+							/* if( !titeInhibit)*/
 							ml_screen_use_normal_edit(
 								vt100_parser->screen) ;
 							restore_cursor( vt100_parser) ;
-							//
+							/* */
 						}
 						else
 						{
@@ -1911,19 +1947,6 @@ parse_vt100_escape_sequence(
 						
 						if( num == 2)
 						{
-							/*
-							 * in case 0 is used.
-							 */
-							if( ps[0] == 0)
-							{
-								ps[0] = 1 ;
-							}
-
-							if( ps[1] == 0)
-							{
-								ps[1] = 1 ;
-							}
-
 							ml_screen_set_scroll_region(
 								vt100_parser->screen ,
 								ps[0] - 1 , ps[1] - 1) ;
@@ -1971,7 +1994,7 @@ parse_vt100_escape_sequence(
 			}
 			else if( *str_p == ']')
 			{
-				u_char  digit[10] ;
+				char  digit[10] ;
 				int  count ;
 				int  ps ;
 				u_char *  pt ;
@@ -2291,7 +2314,7 @@ parse_vt100_escape_sequence(
 		#ifdef  ESCSEQ_DEBUG
 			kik_debug_printf( KIK_DEBUG_TAG " receiving SI\n") ;
 		#endif
-		
+
 			vt100_parser->is_dec_special_in_gl = vt100_parser->is_dec_special_in_g0 ;
 			vt100_parser->is_so = 0 ;
 		}
@@ -2307,7 +2330,7 @@ parse_vt100_escape_sequence(
 		#ifdef  ESCSEQ_DEBUG
 			kik_debug_printf( KIK_DEBUG_TAG " receiving SO\n") ;
 		#endif
-		
+
 			vt100_parser->is_dec_special_in_gl = vt100_parser->is_dec_special_in_g1 ;
 			vt100_parser->is_so = 1 ;
 		}
@@ -2547,15 +2570,13 @@ ml_parse_vt100_sequence(
 			/*
 			 * parsing character encoding.
 			 */
-
 			(*vt100_parser->cc_parser->set_str)( vt100_parser->cc_parser ,
 				CURRENT_STR_P(vt100_parser) , vt100_parser->left) ;
-
 			while( (*vt100_parser->cc_parser->next_char)( vt100_parser->cc_parser , &ch))
 			{
 				/*
 				 * UCS <-> OTHER CS
-				 */ 
+				 */
 				if( ch.cs == ISO10646_UCS4_1)
 				{
 					if( ch.ch[0] == 0x00 && ch.ch[1] == 0x00 &&
@@ -2685,7 +2706,6 @@ ml_parse_vt100_sequence(
 					{
 						SET_MSB( ch.ch[0]) ;
 					}
-
 					if( ( ch.cs == US_ASCII && vt100_parser->is_dec_special_in_gl) ||
 						ch.cs == DEC_SPECIAL)
 					{
@@ -2853,22 +2873,19 @@ ml_init_encoding_conv(
 	 */
 	if( IS_STATEFUL_ENCODING(vt100_parser->encoding))
 	{
-		(*vt100_parser->cc_parser->init)( vt100_parser->cc_parser) ;
-		vt100_parser->is_dec_special_in_gl = 0 ;
-		vt100_parser->is_so = 0 ;
-		vt100_parser->is_dec_special_in_g0 = 0 ;
-		vt100_parser->is_dec_special_in_g1 = 1 ;
+		ml_init_encoding_parser(vt100_parser) ;
 	}
 
 	return  1 ;
 }
 
 int
-ml_vt100_parser_enable_logging_vt_seq(
-	ml_vt100_parser_t *  vt100_parser
+ml_vt100_parser_set_logging_vt_seq(
+        ml_vt100_parser_t *  vt100_parser ,
+        int  flag
 	)
 {
-	vt100_parser->logging_vt_seq = 1 ;
+	vt100_parser->logging_vt_seq = flag ;
 
 	return  1 ;
 }
