@@ -1,5 +1,5 @@
 /*
- *	$Id: ml_vt100_parser.h,v 1.10 2004/12/13 09:31:54 h_minami Exp $
+ *	$Id: ml_vt100_parser.h,v 1.12 2006/01/08 11:46:42 kzys Exp $
  */
 
 #ifndef  __ML_VT100_PARSER_H__
@@ -117,6 +117,7 @@ typedef struct  ml_vt100_parser
 	int8_t  saved_is_bold ;
 	int8_t  saved_is_underlined ;
 	int8_t  saved_is_reversed ;
+	mkf_charset_t  saved_cs ;
 
 } ml_vt100_parser_t ;
 
@@ -151,7 +152,7 @@ int  ml_init_encoding_parser( ml_vt100_parser_t *  vt100_parser) ;
 
 int  ml_init_encoding_conv( ml_vt100_parser_t *  vt100_parser) ;
 
-int  ml_vt100_parser_enable_logging_vt_seq( ml_vt100_parser_t *  vt100_parser) ;
+int  ml_vt100_parser_set_logging_vt_seq( ml_vt100_parser_t *  vt100_parser , int  size) ;
 
 
 #endif
