@@ -1,5 +1,5 @@
 /*
- *	$Id: main.c,v 1.15 2005/02/22 15:09:44 h_minami Exp $
+ *	$Id: main.c,v 1.16 2005/09/23 05:40:20 mfabian Exp $
  */
 
 #include  <stdio.h>
@@ -82,13 +82,13 @@ main(
 			{
 				help() ;
 
-				exit(0) ;
+				return  0 ;
 			}
 			else if( strcmp( p , "version") == 0 || strcmp( p , "v") == 0)
 			{
 				version() ;
 
-				exit(0) ;
+				return  0 ;
 			}
 			else if( strcmp( p , "e") == 0)
 			{
