@@ -1,16 +1,16 @@
 /*
- *	$Id: mc_sb_view.c,v 1.11 2004/09/13 11:06:57 seiichi Exp $
+ *	$Id: mc_sb_view.c,v 1.15 2004/12/06 04:24:41 h_minami Exp $
  */
 
 #include  "mc_sb_view.h"
 
-#include  <string.h>
-#include  <stdlib.h>		/* free */
+#include  <kiklib/kik_str.h>
+#include  <kiklib/kik_mem.h>		/* free */
 #include  <kiklib/kik_debug.h>
 #include  <kiklib/kik_conf_io.h>
 #include  <glib.h>
 #include  <c_intl.h>
-#include  <sys/types.h>
+#include  <kiklib/kik_types.h>
 #include  <dirent.h>
 
 #include  "mc_combo.h"
@@ -32,8 +32,8 @@
 
 /* --- static variables --- */
 
-static char *  new_sb_view_name ;
-static char *  old_sb_view_name ;
+static char *  new_sb_view_name = NULL;
+static char *  old_sb_view_name = NULL;
 static int is_changed;
 
 
@@ -45,7 +45,8 @@ sb_view_name_selected(
 	gpointer  data
 	)
 {
-	new_sb_view_name = gtk_entry_get_text(GTK_ENTRY(widget)) ;
+	free( new_sb_view_name);
+	new_sb_view_name = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1) ;
 	
 #ifdef  __DEBUG
 	kik_debug_printf( KIK_DEBUG_TAG " %s sb_view_name is selected.\n" , new_sb_view_name) ;
@@ -60,17 +61,21 @@ has_rc_file(char *dirname, char *sbdirname)
 	DIR *d;
 	struct dirent *e;
 	char path[PATH_MAX];
+	int result = 0;
 
 	snprintf(path, PATH_MAX, "%s/%s", dirname, sbdirname);
 	if(!(d = opendir(path))) return 0;
 
 	while ((e = readdir(d))) {
 		if (strcmp("rc", e->d_name) == 0) {
-			return 1;
+			result = 1;
+			break;
 		}
 	}
 
-	return 0;
+	closedir(d);
+
+	return result;
 }
 
 static int
@@ -130,8 +135,8 @@ mc_sb_view_config_widget_new(void)
 	if (userdir) n = read_sb_names(userdir, sb_view_names, n);
 	n = read_sb_names(SB_DIR, sb_view_names, n);
 
-	new_sb_view_name = old_sb_view_name =
-	  mc_get_str_value( "scrollbar_view_name") ;
+	new_sb_view_name =
+		strdup( old_sb_view_name = mc_get_str_value( "scrollbar_view_name")) ;
 	is_changed = 0;
 
 	return  mc_combo_new( _("View") , sb_view_names , n,
