@@ -1,9 +1,9 @@
 /*
- *	$Id: x_im.c,v 1.17 2005/02/22 17:00:30 seiichi Exp $
+ *	$Id: x_im.c,v 1.18 2005/09/30 04:13:59 h_minami Exp $
  */
-
+#include  <stdio.h>		/* sprintf */
 #include  <kiklib/kik_mem.h>	/* malloc/alloca/free */
-#include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_str_sep kik_snprintf*/
+#include  <kiklib/kik_str.h>	/* kik_str_alloca_dup kik_str_sep */
 #include  <kiklib/kik_locale.h>
 
 #include  "x_im.h"
