@@ -1,7 +1,8 @@
 /*
- *	$Id: x_im_candidate_screen.c,v 1.8 2004/10/10 12:48:44 seiichi Exp $
+ *	$Id: x_im_candidate_screen.c,v 1.16 2005/03/01 17:56:31 seiichi Exp $
  */
 
+#include  <kiklib/kik_str.h>
 #include  "x_im_candidate_screen.h"
 
 #ifdef  USE_IM_PLUGIN
@@ -77,6 +78,39 @@ free_candidates(
 	return  1 ;
 }
 
+static u_int
+candidate_width(
+	x_font_manager_t *  font_man ,
+	x_im_candidate_t *  candidate
+	)
+{
+	u_int  width ;
+	int  i ;
+
+	if( candidate->chars == NULL || candidate->filled_len == 0)
+	{
+		return  0 ;
+	}
+
+	width = 0 ;
+
+	for( i = 0 ; i < candidate->filled_len ; i++)
+	{
+		x_font_t *  xfont ;
+
+		xfont = x_get_font( font_man ,
+				    ml_char_font( &candidate->chars[i])) ;
+
+		width += x_calculate_char_width(
+					xfont ,
+					ml_char_bytes( &candidate->chars[i]) ,
+					ml_char_size( &candidate->chars[i]) ,
+					ml_char_cs( &candidate->chars[i])) ;
+	}
+
+	return  width ;
+}
+
 static u_int
 max_candidate_width(
 	x_font_manager_t *  font_man ,
@@ -84,37 +118,16 @@ max_candidate_width(
 	u_int  num_of_candidates
 	)
 {
-	x_im_candidate_t *  c ;
 	u_int  max_width ;
 	int  i ;
 
 	max_width = 0 ;
 
-	for( i = 0 , c = candidates ; i < num_of_candidates  ; i++ , c++)
+	for( i = 0 ; i < num_of_candidates  ; i++)
 	{
 		u_int  width ;
-		int  j ;
 
-		if( c->chars == NULL || c->filled_len == 0)
-		{
-			continue ;
-		}
-
-		width = 0 ;
-
-		for( j = 0 ; j < c->filled_len ; j++)
-		{
-			x_font_t *  xfont ;
-
-			xfont = x_get_font( font_man ,
-					    ml_char_font( &c->chars[j])) ;
-
-			width += x_calculate_char_width(
-						xfont ,
-						ml_char_bytes( &c->chars[j]) ,
-						ml_char_size( &c->chars[j]) ,
-						ml_char_cs( &c->chars[j])) ;
-		}
+		width = candidate_width( font_man , &candidates[i]) ;
 
 		if( width > max_width)
 		{
@@ -125,26 +138,41 @@ max_candidate_width(
 	return  max_width ;
 }
 
+static u_int
+total_candidate_width(
+	x_font_manager_t *  font_man ,
+	x_im_candidate_t *  candidates ,
+	u_int  to ,
+	u_int  from
+	)
+{
+	u_int  total_width ;
+	int  i ;
+
+	total_width = 0 ;
+
+	for( i = to ; i <= from  ; i++)
+	{
+		total_width += candidate_width( font_man , &candidates[i]) ;
+	}
+
+	return  total_width ;
+}
+
 #define  MAX_NUM_OF_DIGITS 4 /* max is 9999. enough? */
 
 static void
-draw_screen(
-	x_im_candidate_screen_t *  cand_screen
+draw_screen_vertical(
+	x_im_candidate_screen_t *  cand_screen ,
+	u_int  top ,
+	u_int  last
 	)
 {
 	x_font_t *  xfont ;
 	u_int  i ;
-	u_int  top ;
-	u_int  last ;
 	u_int  num_of_digits ;
 	u_int  win_width ;
 	u_int  win_height ;
-	int  n ;
-
-	VISIBLE_INDEX(cand_screen->num_of_candidates ,
-		      cand_screen->num_per_window ,
-		      cand_screen->index ,
-		      top , last) ;
 
 	if( cand_screen->num_of_candidates > cand_screen->num_per_window)
 	{
@@ -184,8 +212,8 @@ draw_screen(
 	/* width of window */
 	win_width = xfont->width * (num_of_digits + 1) ;
 	win_width += max_candidate_width( cand_screen->font_man ,
-					&cand_screen->candidates[top] ,
-					last - top + 1) ;
+					  &cand_screen->candidates[top] ,
+					  last - top + 1) ;
 	if( win_width < ( MAX_NUM_OF_DIGITS * 2 + 1) * xfont->width)
 	{
 		win_width = ( MAX_NUM_OF_DIGITS * 2 + 1) * xfont->width ;
@@ -234,7 +262,7 @@ draw_screen(
 		 * |1 cand0   |
 		 *  ^
 		 */
-		snprintf( digit , MAX_NUM_OF_DIGITS + 1 , "%i    " , i - top + 1) ;
+		kik_snprintf( digit , MAX_NUM_OF_DIGITS + 1 , "%i    " , i - top + 1) ;
 
 		for( j = 0 ; j < num_of_digits + 1 ; j++)
 		{
@@ -286,7 +314,8 @@ draw_screen(
 	 * |                |/
 	 * +----------------+
 	 */
-	if( cand_screen->num_of_candidates > 10 && last - top < 10)
+	if( cand_screen->num_of_candidates > cand_screen->num_per_window &&
+	    last - top < cand_screen->num_per_window)
 	{
 		u_int  y ;
 
@@ -310,7 +339,7 @@ draw_screen(
 		size_t  len ;
 		int  x ;
 
-		len = snprintf( navi , MAX_NUM_OF_DIGITS * 2 + 2 , "%d/%d",
+		len = kik_snprintf( navi , MAX_NUM_OF_DIGITS * 2 + 2 , "%d/%d",
 				cand_screen->index + 1 ,
 				cand_screen->num_of_candidates) ;
 
@@ -342,6 +371,145 @@ draw_screen(
 	}
 }
 
+static void
+draw_screen_horizontal(
+	x_im_candidate_screen_t *  cand_screen ,
+	u_int  top ,
+	u_int  last
+	)
+{
+	x_font_t *  xfont ;
+	u_int  win_width ;
+	u_int  win_height ;
+	int  i ;
+	int  x = 0 ;
+
+	/*
+	 * resize window
+	 */
+
+	/*
+	 * +-------------------------------------+
+	 * |1:cand0 2:cand1 3:cand4 ... 10:cand9 |
+	 * |                          index/total|
+	 * +-------------------------------------+
+	 */
+
+	xfont = x_get_usascii_font( cand_screen->font_man) ;
+
+	/* width of window */
+	win_width = 0 ;
+	for( i = 1 ; i <= last - top + 1 ; i++)
+	{
+		u_int  num_of_digits ;
+
+		NUM_OF_DIGITS( num_of_digits , i) ;
+		win_width += xfont->width * (num_of_digits + 2) ;
+	}
+	win_width += total_candidate_width( cand_screen->font_man ,
+					    cand_screen->candidates ,
+					    top , last) ;
+	/* height of window */
+	win_height = xfont->height + LINE_SPACE ;
+
+	x_window_resize( &cand_screen->window , win_width , win_height , 0) ;
+
+	/*
+	 * draw border
+	 */
+	x_window_draw_rect_frame( &cand_screen->window , 0 , 0 ,
+				  win_width + MARGIN*2 - 1 ,
+				  win_height + MARGIN*2 - 1);
+
+	for( i = top ; i <= last; i++)
+	{
+		u_int  num_of_digits ;
+		u_char  digit[MAX_NUM_OF_DIGITS + 1] ;
+		int  j ;
+
+		/*
+		 * digits
+		 * +---------------
+		 * |1:cand0 2:cand1
+		 *  ^^
+		 */
+		NUM_OF_DIGITS( num_of_digits , (i - top + 1)) ;
+		kik_snprintf( digit , MAX_NUM_OF_DIGITS + 1 , "%i." , i - top + 1) ;
+
+		for( j = 0 ; j < num_of_digits + 1 ; j++)
+		{
+			ml_char_t  ch ;
+
+			ml_char_init( &ch) ;
+
+			ml_char_set( &ch , &digit[j] , 1 , US_ASCII , 0 , 0 ,
+				     ML_FG_COLOR , ML_BG_COLOR , 0 , 0) ;
+
+			x_draw_str( &cand_screen->window ,
+				    cand_screen->font_man ,
+				    cand_screen->color_man ,
+				    &ch , 1 ,
+				    x , 0 ,
+				    xfont->height + LINE_SPACE ,
+				    xfont->height_to_baseline + LINE_SPACE / 2 ,
+				    LINE_SPACE / 2 ,
+				    LINE_SPACE / 2 + LINE_SPACE % 2) ;
+			x += xfont->width ;
+		}
+
+		/*
+		 * candidate
+		 * +---------------
+		 * |1:cand0 2:cand2
+		 *    ^^^^^
+		 */
+		x_draw_str( &cand_screen->window ,
+			    cand_screen->font_man ,
+			    cand_screen->color_man ,
+			    cand_screen->candidates[i].chars ,
+			    cand_screen->candidates[i].filled_len ,
+			    x , 0 ,
+			    xfont->height + LINE_SPACE ,
+			    xfont->height_to_baseline + LINE_SPACE / 2 ,
+			    LINE_SPACE / 2 ,
+			    LINE_SPACE / 2 + LINE_SPACE % 2) ;
+		x += candidate_width( cand_screen->font_man ,
+				      &cand_screen->candidates[i]) ;
+
+		/*
+		 * +---------------
+		 * |1:cand0 2:cand2
+		 *         ^
+		 */
+		x_window_clear( &cand_screen->window , x , 1 ,
+				xfont->width , win_height - 2) ;
+		x += xfont->width ;
+	}
+}
+
+static void
+draw_screen(
+	x_im_candidate_screen_t *  cand_screen
+	)
+{
+	u_int  top ;
+	u_int  last ;
+
+	VISIBLE_INDEX(cand_screen->num_of_candidates ,
+		      cand_screen->num_per_window ,
+		      cand_screen->index ,
+		      top , last) ;
+
+	if( cand_screen->is_vertical_direction)
+	{
+		draw_screen_vertical( cand_screen , top , last) ;
+	}
+	else
+	{
+		draw_screen_horizontal( cand_screen , top , last) ;
+	}
+}
+
 static void
 adjust_window_position(
 	x_im_candidate_screen_t *  cand_screen ,
@@ -350,6 +518,7 @@ adjust_window_position(
 	)
 {
 	u_int  dh ;
+	u_int  dw ;
 	u_int  top ;
 	u_int  last ;
 	u_int  num_of_digits ;
@@ -359,22 +528,32 @@ adjust_window_position(
 		       cand_screen->index ,
 		       top , last) ;
 
-	if( cand_screen->num_of_candidates > cand_screen->num_per_window)
+	if( cand_screen->is_vertical_direction)
 	{
-		NUM_OF_DIGITS( num_of_digits , cand_screen->num_per_window) ;
+		if( cand_screen->num_of_candidates > cand_screen->num_per_window)
+		{
+			NUM_OF_DIGITS( num_of_digits , cand_screen->num_per_window) ;
+		}
+		else
+		{
+			NUM_OF_DIGITS( num_of_digits , last) ;
+		}
 	}
 	else
 	{
-		NUM_OF_DIGITS( num_of_digits , last) ;
+		num_of_digits = 1 ;
 	}
 
-	if( cand_screen->is_vertical)
+	if( cand_screen->is_vertical_term)
 	{
 		return ;
 	}
 
 	dh = DisplayHeight( cand_screen->window.display ,
 			    cand_screen->window.screen) ;
+	dw = DisplayWidth( cand_screen->window.display ,
+			   cand_screen->window.screen) ;
+
 
 	if( *y + cand_screen->window.height > dh)
 	{
@@ -385,6 +564,11 @@ adjust_window_position(
 	{
 		*x -= x_get_usascii_font( cand_screen->font_man)->width * (num_of_digits + 1) + MARGIN ;
 	}
+
+	if( *x + cand_screen->window.width > dw)
+	{
+		*x = dw - cand_screen->window.width ;
+	}
 }
 
 /*
@@ -396,8 +580,6 @@ delete(
 	x_im_candidate_screen_t *  cand_screen
 	)
 {
-	int  i ;
-
 	free_candidates( cand_screen->candidates ,
 			 cand_screen->num_of_candidates) ;
 
@@ -458,8 +640,6 @@ init_candidates(
 	u_int num_per_window
 	)
 {
-	int  i ;
-
 	if( cand_screen->candidates)
 	{
 		free_candidates( cand_screen->candidates ,
@@ -472,23 +652,16 @@ init_candidates(
 	cand_screen->num_per_window = num_per_window ;
 
 	/* allocate candidates(x_im_candidate_t) array */
-	if( ( cand_screen->candidates = malloc( sizeof( x_im_candidate_t) * cand_screen->num_of_candidates)) == NULL)
+	if( ( cand_screen->candidates = calloc( sizeof( x_im_candidate_t), cand_screen->num_of_candidates)) == NULL)
 	{
 	#ifdef  DEBUG
-		kik_warn_printf( KIK_DEBUG_TAG " malloc failed.\n") ;
+		kik_warn_printf( KIK_DEBUG_TAG " calloc failed.\n") ;
 	#endif
-
+		cand_screen->num_of_candidates = 0 ;
+		cand_screen->num_per_window = 0 ;
 		return  0 ;
 	}
 
-	/* initialize candidate array */
-	for( i = 0 ; i < cand_screen->num_of_candidates ; i++)
-	{
-		cand_screen->candidates[i].chars = NULL ;
-		cand_screen->candidates[i].num_of_chars = 0 ;
-		cand_screen->candidates[i].filled_len = 0 ;
-	}
-
 	cand_screen->index = 0 ;
 
 	return  1 ;
@@ -780,7 +953,8 @@ x_im_candidate_screen_new(
 	x_window_manager_t *  win_man ,
 	x_font_manager_t *  font_man ,
 	x_color_manager_t *  color_man ,
-	int  is_vertical ,
+	int  is_vertical_term ,
+	int  is_vertical_direction ,
 	u_int  line_height_of_screen ,
 	int  x ,
 	int  y
@@ -809,7 +983,8 @@ x_im_candidate_screen_new(
 
 	cand_screen->line_height = line_height_of_screen ;
 
-	cand_screen->is_vertical = is_vertical ;
+	cand_screen->is_vertical_term = is_vertical_term ;
+	cand_screen->is_vertical_direction = is_vertical_direction ;
 
 	if( ! x_window_init( &cand_screen->window , MARGIN * 2 , MARGIN * 2 ,
 			     MARGIN * 2 , MARGIN * 2 , MARGIN * 2 , MARGIN * 2 ,
@@ -889,7 +1064,8 @@ x_im_candidate_screen_new(
 	x_window_manager_t *  win_man ,
 	x_font_manager_t *  font_man ,
 	x_color_manager_t *  color_man ,
-	int  is_vertical ,
+	int  is_vertical_term ,
+	int  is_vertical_direction ,
 	u_int  line_height_of_screen ,
 	int  x ,
 	int  y
