@@ -1,5 +1,5 @@
 /*
- *	$Id: x_sb_view_factory.c,v 1.10 2003/09/30 10:24:13 arakiken Exp $
+ *	$Id: x_sb_view_factory.c,v 1.11 2003/10/11 16:27:54 arakiken Exp $
  */
 
 #include  "x_sb_view_factory.h"
@@ -27,6 +27,9 @@
 #endif
 
 
+typedef  x_sb_view_t * (*x_sb_view_new_func_t)(void) ;
+typedef  x_sb_view_t * (*x_sb_engine_new_func_t)( x_sb_view_conf_t *  conf, int is_transparent) ;
+
 KIK_LIST_TYPEDEF( x_sb_view_conf_t) ;
 
 
