@@ -1,5 +1,5 @@
 /*
- *	$Id: x_term_manager.c,v 1.91 2005/02/28 00:09:16 seiichi Exp $
+ *	$Id: x_term_manager.c,v 1.92 2006/01/08 11:46:42 kzys Exp $
  */
 
 #include  "x_term_manager.h"
@@ -173,7 +173,7 @@ create_term_intern(void)
 
 	if( main_config.logging_vt_seq)
 	{
-		ml_term_enable_logging_vt_seq( term) ;
+		ml_term_set_logging_vt_seq( term , main_config.logging_vt_seq) ;
 	}
 
 	return  term ;
