@@ -1,5 +1,5 @@
 /*
- *	$Id: x_window.c,v 1.76 2005/02/21 14:43:43 h_minami Exp $
+ *	$Id: x_window.c,v 1.78 2006/02/14 14:06:46 h_minami Exp $
  */
 
 /*
@@ -2612,12 +2612,14 @@ x_window_draw_string(
 	XSetForeground( win->display , win->ch_gc , fg_color->pixel) ;
 
 	XDrawString( win->display , win->drawable , win->ch_gc ,
-		x + font->x_off + win->margin , y + win->margin , str , len) ;
+		x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
+		y + win->margin , (char *)str , len) ;
 
 	if( font->is_double_drawing)
 	{
 		XDrawString( win->display , win->drawable , win->ch_gc ,
-			x + font->x_off + win->margin + 1 , y + win->margin , str , len) ;
+			x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
+			y + win->margin , (char *)str , len) ;
 	}
 
 	return  1 ;
@@ -2638,12 +2640,14 @@ x_window_draw_string16(
 	XSetForeground( win->display , win->ch_gc , fg_color->pixel) ;
 
 	XDrawString16( win->display , win->drawable , win->ch_gc ,
-		x + font->x_off + win->margin , y + win->margin , str , len) ;
+		       x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
+		       y + win->margin , str , len) ;
 
 	if( font->is_double_drawing)
 	{
 		XDrawString16( win->display , win->drawable , win->ch_gc ,
-			x + font->x_off + win->margin + 1 , y + win->margin , str , len) ;
+			       x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
+			       y + win->margin , str , len) ;
 	}
 
 	return  1 ;
@@ -2666,12 +2670,14 @@ x_window_draw_image_string(
 	XSetBackground( win->display , win->ch_gc , bg_color->pixel) ;
 
 	XDrawImageString( win->display , win->drawable , win->ch_gc ,
-		x + font->x_off + win->margin , y + win->margin , str , len) ;
+			  x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
+			  y + win->margin , (char *)str , len) ;
 
 	if( font->is_double_drawing)
 	{
 		XDrawString( win->display , win->drawable , win->ch_gc ,
-			x + font->x_off + win->margin + 1 , y + win->margin , str , len) ;
+			     x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
+			     y + win->margin , (char *)str , len) ;
 	}
 
 	return  1 ;
@@ -2694,12 +2700,14 @@ x_window_draw_image_string16(
 	XSetBackground( win->display , win->ch_gc , bg_color->pixel) ;
 
 	XDrawImageString16( win->display , win->drawable , win->ch_gc ,
-		x + font->x_off + win->margin , y + win->margin , str , len) ;
+			    x + (font->is_var_col_width ? 0 : font->x_off) + win->margin ,
+			    y + win->margin , str , len) ;
 
 	if( font->is_double_drawing)
 	{
 		XDrawString16( win->display , win->drawable , win->ch_gc ,
-			x + font->x_off + win->margin + 1 , y + win->margin , str , len) ;
+			       x + (font->is_var_col_width ? 0 : font->x_off) + win->margin + 1 ,
+			       y + win->margin , str , len) ;
 	}
 
 	return  1 ;
@@ -2837,9 +2845,18 @@ x_window_send_selection(
 	}
 	else
 	{
-		XChangeProperty( win->display , req_ev->requestor ,
-			req_ev->property , sel_type ,
-			8 , PropModeReplace , sel_str , sel_len) ;
+		if( req_ev->property == None)
+		{
+			/* An obsolete client may fill None as a property.
+			 * Try to deal with them by using 'target' instead.
+			 */
+			req_ev->property = req_ev->target;
+		}
+		if( req_ev->property != None){
+			XChangeProperty( win->display , req_ev->requestor ,
+				req_ev->property , sel_type ,
+				8 , PropModeReplace , sel_str , sel_len) ;
+		}
 		res_ev.xselection.property = req_ev->property ;
 	}
 
