@@ -1,5 +1,5 @@
 /*
- *	$Id: x_window.h,v 1.32 2004/10/24 07:12:12 seiichi Exp $
+ *	$Id: x_window.h,v 1.36 2005/03/02 14:13:46 seiichi Exp $
  */
 
 #ifndef  __X_WINDOW_H__
@@ -31,7 +31,7 @@ typedef enum  x_event_dispatch
 	NOTIFY_TO_NONE = 0x0 ,
 	NOTIFY_TO_CHILDREN = 0x01 ,
 	NOTIFY_TO_PARENT = 0x02 ,
-	NOTIFY_TO_MYSELF = 0x04 ,
+	NOTIFY_TO_MYSELF = 0x04
 
 } x_event_dispatch_t ;
 
@@ -128,6 +128,14 @@ typedef struct  x_window
 	 */
 	char *  app_name ;
 
+	
+	/*
+	 * per-window icon
+	 */
+	Pixmap icon_pix;
+	Pixmap icon_mask;
+	u_int32_t *  icon_card;
+	
 	/*
 	 * flags etc.
 	 */
@@ -316,6 +324,10 @@ int  x_set_icon_name( x_window_t *  win , u_char *  name) ;
 
 int  x_window_set_icon( x_window_t *  win , Pixmap  icon , Pixmap  mask , u_int32_t *  cardinal) ;
 
+int x_window_remove_icon( x_window_t *  win) ;
+
+int  x_window_set_icon_from_file( x_window_t *  win , char *  path) ;
+
 int  x_window_get_visible_geometry( x_window_t *  win ,
 	int *  x , int *  y , int *  my_x , int *  my_y , u_int *  width , u_int *  height) ;
 
